/***************************************************************************/
/***                                                                     ***/
/***                  splitinfo.c                                        ***/
/***                                                                     ***/
/***       Ronny Fehling, Bernhard Nebel, Jochen Renz  - March 1998      ***/
/***                                                                     ***/
/***         fehling, nebel, renz@informatik.uni-freiburg.de             ***/
/***                                                                     ***/
/***          http://www.informatik.uni-freiburg.de/~sppraum             ***/
/***                                                                     ***/
/***                      Institut fuer Informatik                       ***/
/***                     Albert-Ludwigs-Universitaet                     ***/
/***                           Am Flughafen 17                           ***/
/***                       79110 Freiburg, Germany                       ***/
/***                                                                     ***/
/***************************************************************************/


/*
    splitinfo coverfile < CSP-file

    reads CSPs from standard input and computes split factor 
    (for non-universal relations) for the coverfile given.
*/


#include "rcc8.h"
#include "rcc8io.h"


const char *usagestring = "\
usage: splitinfo coverfile < CSP-file

    reads CSPs from standard input and computes split factor 
    (for non-universal relations) for the coverfile given.";


RELTYPE splitinfo[MAXSET][9];
int ccountall;
int ccount[9];



void usage()
{
  fprintf(stderr,"%s\n",usagestring);
  exit(-1);
}



void readsplitfile(char *s)
{
  int i,j,inp;
  FILE *splitfile;

  if ((splitfile = fopen(s,"r")) == NULL) {
    fprintf(stderr, "***Relation-cover file '%s' not found\n",s);
    exit(-1);
  }
  for (i=0;i<MAXSET;i++) 
    for (j=0; j<9; j++) {
      if (fscanf(splitfile,"%d",&inp) != 1) {
	fprintf(stderr, "***Format error in relation-cover file '%s'\n",s);
	exit(-1);
      } else {
	splitinfo[i][j] = inp;
      }
    }
  if ((fscanf(splitfile,"%d",&inp) == 1) || (!feof(splitfile))) {
	fprintf(stderr, "***No eof in relation-cover file '%s'\n",s);
      	exit(-1);
      }
}



void printstat()
{
  fprintf(stdout,"Coverage:\n");
  fprintf(stdout,"  1: %4d\n  2: %4d\n  3: %4d\n  4: %4d\n  5: %4d\n  6: %4d\n  7: %4d\n  8: %4d\n",
	  ccount[0],ccount[1],ccount[2],ccount[3],ccount[4],
	  ccount[5],ccount[6],ccount[7]);
  fprintf(stdout,"  average branching: %8.3f\n",((double)(ccount[0]*1+
							   ccount[1]*2+
							   ccount[2]*3+
							   ccount[3]*4+
							   ccount[4]*5+
							   ccount[5]*6+
							   ccount[6]*7+
							   ccount[7]*8)/
						  (double)ccountall));
}


 
void main(int argc, char **argv)
{
  int i,j,maxnode;
  RELTYPE r;
  static char csptype[100];
  static RELTYPE csp[MAXCSP][MAXCSP];

  if (argc != 2) usage();
  readsplitfile(argv[1]);
  ccountall = 0;
  for (i=0; i<8; i++) ccount[i] = 0;
  while (parsecsp(stdin, MAXCSP-1, &maxnode, csptype, csp) == 1) {
    for (i=0; i<=maxnode; i++)
      for (j=i+1; j<= maxnode; j++) {
	if ((r=csp[i][j]) != DALL) {
	  ccount[splitinfo[r][0]-1]++;
	  ccountall++;
	}
      }
  }
  printstat();
}
